function g = kernelGradient(params, model, prior)

% KERNELGRADIENT Gradient of likelihood approximation wrt kernel parameters.
%
% g = kernelGradient(params, model, prior)

% Copyright (c) 2004 Neil D. Lawrence
% File version 1.3, Wed Jun 16 11:02:41 2004
% KERN toolbox version 0.11



if nargin < 3
  prior = 1;
end

x = model.X(model.I, :);
m = model.m(model.I, :);
model.kern = kernExpandParam(model.kern, params);
K = kernCompute(model.kern, x);
g = zeros(size(params));

if model.noise.spherical
  % there is only one value for all beta
  invK = pdinv(K+diag(1./model.beta(model.I, 1)));
end

for j = 1:size(m, 2)
  if ~model.noise.spherical
    invK = pdinv(K+diag(1./model.beta(model.I, j)));
  end
  covGrad = feval([model.type 'CovarianceGradient'], invK, m(:, j));
  g = g + kernGradient(model.kern, x, covGrad);
end  
g = g + kernPriorGradient(model.kern);
g = -g;

